import { retrieveOrGenerateKeyPair } from "../utils/crypto.ts";

const URL_INPUT_ID = "request-url";
const TEMPLATE = `
      <details open>
        <summary>How To Use:</summary>
        <ul>
        <li>Share the URL below with whom you'd like to request data from.</li>
        <li>When the recipient opens the URL, they will be able to enter the data they'd like to send to you.</li>
        <li>They will then generate a new URL with their encrypted data, which only your original device can decrypt.</li>
        </ul>
      </details>

      <form>
        <input
          type="text"
          name="request-url"
          id="${URL_INPUT_ID}"
          placeholder=""
          aria-label="Request URL"
          required
        />
      </form>
    `;

export async function setupRequestPage(element: HTMLElement) {
  element.innerHTML = TEMPLATE;

  const url = await generateUrl();
  const input = document.getElementById(URL_INPUT_ID) as HTMLInputElement;
  input.value = url.toString();
}

async function generateUrl(): Promise<URL> {
  const { publicKey: ecdhPublic } = await retrieveOrGenerateKeyPair();

  // Generate URL with public key as the 'p' search parameter
  const ecdhPublicJwk = await window.crypto.subtle.exportKey("jwk", ecdhPublic);
  const url = new URL(window.location.toString());
  url.hash = `p=${btoa(JSON.stringify(ecdhPublicJwk))}`;

  // Return the generated URL
  return url;
}
